#ifndef YAMANFIFOS8T698698
#define YAMANFIFOS8T698698

#include <boost/thread.hpp>
#include <queue>

#include "log.h"

template <typename T>
class Fifo {
public :
    virtual void push(T toPush) = 0;
    virtual const T& pop() = 0;
    virtual Fifo<T>* createOneOfThisType() = 0;
};

template <typename T, T DEFAULTVALUE>
class FifoStlQueue : public Fifo<T>
{
protected :
    std::queue<T> intern;
    boost::mutex l;
public :
    virtual void push(T toPush)
    {
        l.lock();
        intern.push(toPush);
        l.unlock();
    }
    virtual const T& pop()
    {
        T toReturn = DEFAULTVALUE;
        l.lock();
        if (!intern.empty())
        {
            toReturn = intern.front();
            intern.pop();
        }
        l.unlock();
        return toReturn;
    }
    virtual Fifo<T>* createOneOfThisType()
    {
        return new FifoStlQueue<T,DEFAULTVALUE>;
    }
};

template <typename T, T DEFAULTVALUE, int SIZE>
class FifoCirclingArray : public Fifo<T>
{
public :
    FifoCirclingArray() : nextIn(0),nextOut(0){};
protected :
    T datas[SIZE];
    int nextIn;
    int nextOut;
public :
    virtual void push(T toPush)
    {
        datas[nextIn%SIZE] = toPush;
        nextIn++;
    }
    virtual const T& pop()
    {
        T toReturn = DEFAULTVALUE;
        if (nextOut < nextIn)
        {
            toReturn = datas[nextOut%SIZE];
            nextOut ++;
        }
        return toReturn;
    }
    virtual Fifo<T>* createOneOfThisType()
    {
        return new FifoCirclingArray<T,DEFAULTVALUE,SIZE>;
    }
};

#endif //YAMANFIFOS8T698698
