#ifndef SINGLEGROUPFIFOTESTER_H
#define SINGLEGROUPFIFOTESTER_H

#include "WritingThreadFunctor.h"
#include "ReadingThreadFunctor.h"
#include "fifos.h"
#include "FifoFactory.h"
#include "ResultSet.h"

#include <boost/thread.hpp>
#include <boost/shared_ptr.hpp>
#include <list>

class SingleGroupFifoTester
{
    public:
        SingleGroupFifoTester   (int WTC, int RTC, boost::barrier& goFlag, boost::barrier& stopFlag,
                                    boost::barrier& waitToEnd, bool& writingInterruptorFlag, bool& readerInterruptorFlag,
                                    int waitingTimeAfterWriteMillisec, int waitingTimeIfFifoIsEmptyMillisec);
        virtual ~SingleGroupFifoTester();
        void clean();
        void createFifo(FifoFactory<int>::FifoTypes fifoType);
        void createThrowingThreads();
        void createReadingThreands();
        boost::shared_ptr<ResultSet> compileData();

    protected:
        Fifo<int>* fifo;
        int writingThreadNumber;
        int readingThreadNumber;
        boost::barrier& goFlag;
        boost::barrier& stopFlag;
        boost::barrier& waitToEnd;
        bool& writingInterruptorFlag;
        bool& readerInterruptorFlag;
        std::list<boost::thread*> allWritingThreads;
        std::list<WritingThreadFunctor*> allWritingThreadFunctors;
        std::list<boost::thread*> allReadingThreads;
        std::list<ReadingThreadFunctor*> allReadingThreadFunctors;
        int waitingTimeAfterWriteMillisec;
        int waitingTimeIfFifoIsEmptyMillisec;
};

#endif // SINGLEGROUPFIFOTESTER_H
